(*          file: g5.ml   generates 1^n2^n3^n4^n5^n as an s
        created: Thu Feb 28 09:17:43 PST 2008
   first author: E Stabler stabler@ucla.edu

   Example grammar given by (i) feature to string translations (moverString, catString)
                            (ii) size parameters (moverMax,catMax)
                            (iii) the list of lexical items, and 
                            (iv) some examples (at least zero of them! so that the function is defined). 

   We could write a program to generate the numbering of categories 
   and these other parameters, but for now we do it by hand.

   This grammar builds big charts -- good benchmarker (use native code compilation!)
*)

let moverString f = List.assoc f [(0,"d"); (1,"c"); (2,"b"); (3,"a"); (4,"f"); ];;
let moverMax = 4;;

let catString f = List.assoc f [(0,"f"); (1,"d"); (2,"c"); (3,"b"); (4,"t3"); (5,"t2"); (6,"t1"); (7,"t4"); (8,"a"); (9,"s"); ];;
let catMax = 9;;

let grammar = [
   ([], [Cat 9; ]); 
   ([], [Sel 8; Pos 4; Cat 7; ]); 
   ([], [Sel 6; Pos 3; Cat 9; ]); 
   ([], [Sel 5; Pos 2; Cat 6; ]); 
   ([], [Sel 4; Pos 1; Cat 5; ]); 
   ([], [Sel 7; Pos 0; Cat 4; ]); 
   (["1"], [Sel 3; Cat 8; Neg 3; ]); 
   (["1"], [Sel 3; Pos 3; Cat 8; Neg 3; ]); 
   (["2"], [Sel 2; Cat 3; Neg 2; ]); 
   (["2"], [Sel 2; Pos 2; Cat 3; Neg 2; ]); 
   (["3"], [Sel 1; Cat 2; Neg 1; ]); 
   (["3"], [Sel 1; Pos 1; Cat 2; Neg 1; ]); 
   (["4"], [Sel 0; Cat 1; Neg 0; ]); 
   (["4"], [Sel 0; Pos 0; Cat 1; Neg 0; ]); 
   (["5"], [Cat 0; Neg 4; ]); 
   (["5"], [Sel 8; Pos 4; Cat 0; Neg 4; ]); 
];;

(* apply cky to example with: eg grammar 1 *)
let example = function  (* (startCategory, inputString) pairs *)
    1 -> (9,["1";"2";"3";"4";"5"])
  | 2 -> (9,["1";"1";"2";"2";"3";"3";"4";"4";"5";"5"])
  | 3 -> (9,["1";"1";"1";"2";"2";"2";"3";"3";"3";"4";"4";"4";"5";"5";"5"])
  | 4 -> (9,["1";"1";"1";"1";"2";"2";"2";"2";"3";"3";"3";"3";"4";"4";"4";"4";"5";"5";"5";"5"])
  | -1 -> (9,["1";"1";"2";"3";"4";"5"])
  | _ -> raise (failwith "No such example");;
